import meteva
import numpy as np
import matplotlib
import matplotlib.image as image
from matplotlib import cm
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import pkg_resources
import math
import os
import colorsys
from matplotlib.colors import BoundaryNorm

def clev_cmap_temper_2m_k():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_temp_2m.txt")
    cmap,clevs =  get_cmap_and_clevs_from_file(path)
    clevs += 273.15
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs,cmap

def clev_cmap_temper_2m():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_temp_2m.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap

def clev_cmap_rain_1h():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_1h.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_rain_3h():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_3h.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_rain_24h():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_24h.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_rh():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rh.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_vis():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_vis.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_wind_speed():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_wind_speed.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_cloud_total():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_tcdc.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap

def clev_cmap_rain_1h_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_1h_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_rain_3h_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_3h_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_rh_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rh_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_vis_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_vis_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_wind_speed_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_wind_speed_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap
def clev_cmap_cloud_total_error():
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_tcdc_error.txt")
    cmap, clevs = get_cmap_and_clevs_from_file(path)
    print("不再推荐使用该函数，推荐使用meb.def_cmap_clevs相应功能,请参考color工具中相关说明")
    return clevs, cmap


def cmap_clevs_temper_2m_error():
    clevs1 = [-20,-12,-8,-6,-4,-2,-1]
    nclev = len(clevs1)
    colors0 = cm.get_cmap("winter", nclev)
    colors_list = []
    for i in range(nclev):
        colors_list.append(colors0(i))

    clevs2 = [0,1,2,4,6,8,12,20]
    nclev = len(clevs2)
    colors0 = cm.get_cmap("autumn", nclev)
    for i in range(nclev):
        colors_list.append(colors0(nclev - 1 - i))
    clevs1.extend(clevs2)
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap, clevs1


def cmap_clevs_environment():
    clevs1 =[0,35,75,115,150,250,350]
    nclev = len(clevs1)
    colors_list = np.array([[0,228,0],[255,255,0],[255,126,0],[255,0,0],[153,0,76],[126,0,35]])/256
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap, clevs1

def get_cmap_and_clevs_from_file(path):
    clev_cmap = np.loadtxt(path)
    clevs = clev_cmap[:, 0]
    cmap = clev_cmap[:, 1:] / 255
    cmap = cmap.tolist()
    cmap = colors.ListedColormap(cmap, 'indexed')
    return cmap,clevs

def cmap_clevs_bias_(vmax):
    if vmax is None:
        print("设置bias的cmap时需要指定vmax")
    blue = np.array([0, 0, 255]) / 255
    white = np.array([255, 255, 255]) / 255
    red = np.array([255, 0, 0]) / 255
    black = np.array([0, 0, 0]) / 255
    clev_list = [0]
    cmap_list = [blue]
    for v in range(1,6):
        clev_list.append(v * 0.2)
        cmap_list.append(blue * (1 - v * 0.2) + white * v * 0.2)
    for v in range(1,6):
        clev_list.append(1 + v * 0.2)
        cmap_list.append(white * (1 - v * 0.2) + red * v * 0.2)

    for value in range(2, int(vmax + 1), 1):
        clev_list.append(value)
        cmap_list.append((red * (vmax - value) + black * (value - 2)) / (vmax - 2))
    cmap = colors.ListedColormap(cmap_list, 'indexed')
    return cmap,clev_list


def cmap_clevs_bias(vmax):
    if vmax is None:
        print("设置bias的cmap时需要指定vmax")
    blue = np.array([0, 0, 255]) / 255
    white = np.array([255, 255, 255]) / 255
    yellow = np.array([0.7,0.7,0])
    red = np.array([255, 0, 0]) / 255
    pink = np.array([0.5,0,0.5])
    black = np.array([0, 0, 0]) / 255
    clev_list = [0]
    cmap_list = [blue]
    for v in range(1,6):
        clev_list.append(v * 0.2)
        cmap_list.append(blue * (1 - v * 0.2) + white * v * 0.2)

    for v in range(1,6):
        clev_list.append(1 + v * 0.2)
        cmap_list.append(white * (1 - v * 0.2) + yellow * v * 0.2)

    for v in range(1,6):
        clev_list.append(2 + v * 0.2)
        cmap_list.append(yellow * (1 - v * 0.2) + red * v * 0.2)

    for v in range(1,10):
        clev_list.append(3 + v * 0.5)
        cmap_list.append(red * (1 - v * 0.1) + pink * v * 0.1)

    for value in range(8, int(vmax + 1), 1):
        clev_list.append(value)
        cmap_list.append((pink * (vmax - value) + black * (value - 2)) / (vmax - 2))
    cmap = colors.ListedColormap(cmap_list, 'indexed')
    return cmap,clev_list


def cmap_clevs_temper_error_br(vmax):
    blue = np.array([0, 0, 255]) / 255
    white = np.array([255, 255, 255]) / 255
    red = np.array([255, 0, 0]) / 255
    black = np.array([0, 0, 0]) / 255
    clev_list = []
    cmap_list = []

    vmax = int(math.ceil(vmax))

    if vmax > 5:
        for value in range(-vmax, -5):
            clev_list.append(value)
            cmap_list.append((blue * (value + vmax) + black * (-5 - value)) / (vmax -5))

    for v in range(-5,0):
        clev_list.append(v)
        cmap_list.append((blue * (-v) + white * (5+v))/5)
    for v in range(0,6):
        clev_list.append(v)
        cmap_list.append((white * (5 - v) + red*v)/5)

    if vmax>5:
        for value in range(6, vmax):
            clev_list.append(value)
            cmap_list.append((red * (vmax - value) + black * (value-5)) / (vmax - 5))

    cmap = colors.ListedColormap(cmap_list, 'indexed')
    return cmap,clev_list


def cmap_clevs_me(vmin,vmax):
    max_abs = max(abs(vmax),abs(vmin))
    vmax = max_abs
    vmin = -max_abs
    dif = (vmax - vmin) / 10.0
    inte = math.pow(10, math.floor(math.log10(dif)));
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8

    inte = inte/2

    vmin = inte * ((int)(vmin / inte) - 1)

    vmax = inte * ((int)(vmax / inte) + 2)
    clevs1 = np.arange(vmin, -1e-6, inte)
    nclev = len(clevs1)
    colors0 = cm.get_cmap("winter", nclev)
    colors_list = []
    for i in range(nclev):
        colors_list.append(colors0(i))

    clevs2 = np.arange(0, vmax, inte)
    nclev = len(clevs2)
    colors0 = cm.get_cmap("autumn", nclev)
    for i in range(nclev):
        colors_list.append(colors0(nclev -1 - i))
    clevs = np.arange(vmin, vmax, inte)
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs


def cmap_clevs_me_w0(vmin,vmax):
    max_abs = max(abs(vmax),abs(vmin))
    vmax = max_abs
    vmin = -max_abs
    dif = (vmax - vmin) / 10.0
    inte = math.pow(10, math.floor(math.log10(dif)));
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8

    inte = inte/2

    vmin = inte * int(math.floor((vmin / inte)))

    vmax = inte * ((int)(vmax / inte) + 1)

    clevs = np.arange(vmin, vmax, inte)
    num = len(clevs)
    rgb_colors = []
    sp = 3/4
    step = 360*sp / num
    #print(num)
    delta0 = int((num-1)/2)
    left = delta0 % 4
    for i in range(num):
        delta = abs(i-((num-1)/2))
        if i < (num-1)/2:
            h = 0.25 +(1-sp)/2 + delta * step / 360  # 首先均匀的取不同的色相，保持色相维度的差异最大化
            if delta <=(left + 4):
                if delta <=1:delta = 0
                s = 0.75 + 0.25 * delta / (left + 4)  # 通过一个折线波浪 设置不同的饱和度
                l = 0.9 - 0.6 * np.power(delta / (left + 4),1)  # 通过一个折线波浪 设置不同的亮度
            else:
                i1 = i % 4
                s = 0.75 + 0.25 * (i1 / 4)  # 通过一个折线波浪 设置不同的饱和度
                l = 0.3 + 0.6 * np.power((i1 / 4),1)  # 通过一个折线波浪 设置不同的亮度

        else:
            h = 0.25 - (1 - sp) / 2 - delta * step / 360  # 首先均匀的取不同的色相，保持色相维度的差异最大化
            if delta < (left + 4):
                s = 0.75 + 0.25 * delta / (left + 4)  # 通过一个折线波浪 设置不同的饱和度
                l = 0.9 - 0.6 * np.power(delta / (left + 4),1)  # 通过一个折线波浪 设置不同的亮度
            else:
                i1 = 4 - (num-1 - i) % 4
                s = 1 - 0.25 * i1 / 4  # 通过一个折线波浪 设置不同的饱和度
                l = 0.75 - 0.35 * np.power(i1 / 4,1)  # 通过一个折线波浪 设置不同的亮度



        rgb1 = colorsys.hls_to_rgb(h, l, s)
        rgb_colors.append(rgb1)

    clevs = np.arange(vmin, vmax, inte)
    cmap = colors.ListedColormap(rgb_colors, 'indexed')
    return cmap,clevs

def cmap_clevs_me_new(vmin,vmax):
    max_abs = max(abs(vmax),abs(vmin))
    vmax = max_abs
    vmin = -max_abs
    dif = (vmax - vmin) / 10.0
    inte = math.pow(10, math.floor(math.log10(dif)));
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8

    inte = inte/2

    vmin = inte * ((int)(vmin / inte) - 1)

    vmax = inte * ((int)(vmax / inte) + 2)
    clevs1 = np.arange(vmin, -1e-6, inte)
    nclev = len(clevs1)
    colors0 = cm.get_cmap("winter", nclev)
    colors_list = []
    for i in range(nclev):
        if i == nclev - 1:
            c1 = np.array(list(colors0(i)))/5 +4/5
            colors_list.append(c1)
        else:
            colors_list.append(colors0(i))

    clevs2 = np.arange(0, vmax, inte)
    nclev = len(clevs2)
    colors0 = cm.get_cmap("autumn", nclev)
    for i in range(nclev):
        if i ==0:
            c1 = np.array(list(colors0(nclev -1 - i)))/5 +4/5
            colors_list.append(c1)
        else:
            colors_list.append(colors0(nclev -1 - i))

    clevs = np.arange(vmin, vmax, inte)
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs


def cmap_clevs_mae(vmax):

    dif = (vmax) / 10.0
    if dif <=0:dif = 1
    inte = math.pow(10, math.floor(math.log10(dif)))
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8

    inte = inte / 2

    vmin = 0

    vmax = inte * ((int)(vmax / inte) + 1)

    clevs = np.arange(vmin, vmax, inte)
    num = len(clevs)
    rgb_colors = []

    for i in range(num):
        h = 0.8 - i/num
        s = 1
        l = 1 - 0.6 * (i+1)/num
        rgb1 = colorsys.hls_to_rgb(h, l, s)
        rgb_colors.append(rgb1)

    clevs = np.arange(vmin, vmax, inte)
    cmap = colors.ListedColormap(rgb_colors, 'indexed')
    return cmap, clevs


def cmap_clevs_me_bwr(vmin,vmax):

    # max_abs = math.ceil(max(abs(vmax),abs(vmin)))
    # inte = 1
    # vmin = inte * ((int)(vmin / inte))
    #
    max_abs = max(abs(vmax),abs(vmin))
    vmax = max_abs
    vmin = -max_abs
    dif = (vmax - vmin) / 10.0
    inte = math.pow(10, math.floor(math.log10(dif)));
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8

    inte = inte/2

    vmin = inte * ((int)(vmin / inte) - 1)
    vmax = inte * ((int)(vmax / inte) + 2)
    max_abs_h = inte * math.ceil(max(abs(vmax),abs(vmin))/inte)/2
    clevs = []
    colors_list = []

    for i in np.arange(vmin,-1e-10,inte).tolist():
        clevs.append(round(i,10))
        if i <= -max_abs_h:
            rgb = [0,0,(i - vmin)/max_abs_h]
        else:
            rgb = [1+ i/max_abs_h,1+ i/max_abs_h,1]
        #print(rgb)
        if i>=-inte*1.5:
            rgb= [1,1,1]
        colors_list.append(rgb)

    for i in np.arange(0,vmax,inte).tolist():
        clevs.append(i)
        if i <= max_abs_h:
            rgb = [1,1- i/max_abs,1- i/max_abs]
        else:
            rgb = [(vmax - i)/max_abs_h,0,0]
        if i < inte:
            rgb = [1,1,1]
        colors_list.append(rgb)
    #print(colors_list)
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs


def cmap_clevs_mode(vmax):
    '''
    定义mode目标绘制的colorbar
    :param vmax:  最大的目标编号
    :return:
    '''
    if vmax <12:
        cmap1,clevs1 = def_cmap_clevs(cmap="Paired",clevs=np.arange(1,vmax+2))
    elif vmax <20:
        cmap1, clevs1 = def_cmap_clevs(cmap="tab20b", clevs=np.arange(1, vmax+1))
    else:
        cmap1, clevs1 = def_cmap_clevs(cmap="gist_rainbow", clevs=np.arange(1, vmax+1))

    #cmap2 = colors.ListedColormap([[80,80,80],[125,125,125],[255,255,255]], 'indexed')
    #clevs2 = [-2,-1,0]
    cmap2,clevs2 = def_cmap_clevs(cmap = "gray",clevs = [-2,-1,0],vmin =-1,vmax = 0,cut_colorbar=True)
    cmap2,clevs2 = get_part_cmap_and_clevs(cmap2,clevs2,vmin= -0.5,vmax = 0,cut_accurate=True)
    cmap3,clevs3 = merge_cmap_clevs(cmap2,clevs2,cmap1,clevs1)
    clevs3 = (np.array(clevs3) -0.5).tolist()

    #show_cmap_clev(cmap3,clevs3)
    return cmap3,clevs3

def cmap_clevs_ts():
    clevs = np.arange(0,1.01,0.1)
    nclev = len(clevs)
    colors0 = cm.get_cmap("jet", nclev)
    colors_list = []
    for i in range(nclev):
        clev = clevs[i]
        cl = []
        cl[0:3] = colors0(i)[0:3]
        cl.append(clev)
        colors_list.append(tuple(cl))
        #print(colors0(i))
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs

def hour():
    #import matplotlib.colors as colors
    cmap = np.array([[0,0,255],
                     [0,115,255],
                     [16,190,56],
                     [255,255,0],
                     [254,147,5],
                     [255,18,0],
                     [177,0,62],
                     [117,0,121],
                     [0,0,255],])/255
    cmap = cmap.tolist()
    cmap = colors.ListedColormap(cmap, 'indexed')
    clevs = np.arange(0,24,3).tolist()
    return  cmap ,clevs


def cmap_clevs_radar():
    clevs = np.arange(5,75,5).tolist()
    clevs1 = [-5,0]
    clevs1.extend(clevs)
    colors_list = np.array([
        [255, 255, 255],
        [0,0,246],
        [1,160,246],
        [78,242,242],
        [1,255,0],
        [0,200,0],
        [1,144,0],
        [255,255,0],
        [231,192,0],
        [255,144,0],
        [255,0,0],
        [214,0,0],
        [192,0,0],
        [255,0,240],
        [120,0,132],
        [173,144,240]
    ])/256
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs1

def cmap_clevs_far():
    clevs = np.arange(0,1.01,0.1)
    nclev = len(clevs)
    colors0 = cm.get_cmap("jet", nclev)
    colors_list = []
    for i in range(nclev):
        colors_list.append(colors0(i))
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs

def cmap_clevs_mr():
    clevs = np.arange(0,1.01,0.1)
    nclev = len(clevs)
    colors0 = cm.get_cmap("jet", nclev)
    colors_list = []
    for i in range(nclev):
        colors_list.append(colors0(i))
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs


def cmap_clevs_error(vmin,vmax):

    max_abs = max(abs(vmax),abs(vmin))
    vmax = max_abs
    vmin = -max_abs
    dif = (vmax - vmin) / 10.0
    inte = math.pow(10, math.floor(math.log10(dif)));
    # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
    r = dif / inte
    if r < 3 and r >= 1.5:
        inte = inte * 2
    elif r < 4.5 and r >= 3:
        inte = inte * 4
    elif r < 5.5 and r >= 4.5:
        inte = inte * 5
    elif r < 7 and r >= 5.5:
        inte = inte * 6
    elif r >= 7:
        inte = inte * 8
    vmin = inte * ((int)(vmin / inte) - 1)
    vmax = inte * ((int)(vmax / inte) + 2)
    clevs = np.arange(vmin, vmax, inte)
    nclev = len(clevs)
    colors0 = cm.get_cmap("bwr", nclev)
    colors_list = []
    for i in range(nclev):
        colors_list.append(colors0(i))
    cmap = colors.ListedColormap(colors_list, 'indexed')
    return cmap,clevs


def get_steps_range(line):
    num = len(line)
    max_nstep = 0

    #首先获得line中各段的起始坐标和长度
    start_list = [0]
    lenght_list = []
    n = 0
    for k in range(1,num):
        n += 1
        if line[k] != line[k-1]:
            lenght_list.append(n)
            start_list.append(k)
            n = 0
    lenght_list.append(n)

    #统计长度序列中保持不变的部分
    start_list_length = [0]
    lenght_list_lenght = []
    n = 0
    for k in range(1,len(lenght_list)):
        n += 1
        if abs(lenght_list[k] - lenght_list[k-1]) > 1:
            lenght_list_lenght.append(n)
            start_list_length.append(k)
            n = 0
    lenght_list_lenght.append(n)

    #计算总共有多少个color等级
    step_num  =  np.max(np.array(lenght_list_lenght))
    #计算等长序列最大个数在lenght_list_lenght对应的起始位置
    k = np.argmax(np.array(lenght_list_lenght))

    # 计算lenght_list_lenght起始坐标在lenght_list中的位置
    j = start_list_length[k]

    # lenght_list中的位置在整个line中的位置
    i_start_list = []
    for i in range(step_num + 1):
        i_start_list.append(start_list[j + i])

    return step_num,i_start_list

def get_cmap_from_picture(path,show = False):

    #im的第一维是y方向，第二维是x方向，第三维是rgb
    im = image.imread(path)
    #首先沿着x方向寻找
    rgb_to_1d = np.zeros((im.shape[0],im.shape[1]))
    rgb_to_1d = im[:,:,0] * 256* 256 + im[:,:,1] * 256 + im[:,:,2]


    color_type_num_y = np.zeros(im.shape[0])
    for y in range(im.shape[0]):
        line = rgb_to_1d[y,:] #line 是一条沿着x方向的线
        rgb_list = line.tolist()
        color_set  = set(rgb_list)
        color_type_num_y[y] = len(color_set)

    color_type_num_x = np.zeros(im.shape[1])
    for x in range(im.shape[1]):
        line = rgb_to_1d[:,x] #line_x 是一条沿着y方向的线
        rgb_list = line.tolist()
        color_set  = set(rgb_list)
        color_type_num_x[x] = len(color_set)


    max_color_type_y = np.max(color_type_num_y)
    max_color_type_x = np.max(color_type_num_x)
    max_color_type = max(max_color_type_x,max_color_type_y)


    max_step_num = 0
    i_start = 0
    i_end = 0
    j_start  = 0
    j_end = 0
    color_list = []

    ij_list = np.where(color_type_num_x == max_color_type)[0]
    if(len(ij_list) > 0):
        k_list_list = []
        k_list = [0]
        for k in range(1,len(ij_list)):
            if ij_list[k] - ij_list[k-1] == 1:
                k_list.append(k)
            else:
                k_list_list.append(k_list)
                k_list = [k]
        k_list_list.append(k_list)

        for k in range(len(k_list_list)):
            k_list =k_list_list[k]
            mid = int((ij_list[0] + ij_list[-1])/2)
            line = rgb_to_1d[:,mid]
            step_num,start_list = get_steps_range(line)
            if step_num > max_step_num:
                max_step_num = step_num
                i_start = ij_list[0]
                i_end = ij_list[-1]
                j_start = start_list[0]
                j_end = start_list[-1]
                color_list = []
                for i in range(step_num):
                    start = start_list[-i-2]
                    color_list.append(im[start,mid,:])


    #
    ij_list = np.where(color_type_num_y == max_color_type)[0]
    if len(ij_list) >0:
        k_list_list = []
        k_list = [0]
        for k in range(1,len(ij_list)):
            if ij_list[k] - ij_list[k-1] == 1:
                k_list.append(k)
            else:
                k_list_list.append(k_list)
                k_list = [k]
        k_list_list.append(k_list)

        for k in range(len(k_list_list)):
            k_list =k_list_list[k]
            mid = int((ij_list[0] + ij_list[-1])/2)
            line = rgb_to_1d[mid,:]
            step_num,start_list = get_steps_range(line)
            if step_num > max_step_num:
                max_step_num = step_num
                j_start = ij_list[0]
                j_end = ij_list[-1]
                i_start = start_list[0]
                i_end = start_list[-1]
                color_list = []
                for i in range(step_num):
                    start = start_list[i]
                    color_list.append(im[mid,start,:])

    cmap = colors.ListedColormap(color_list, 'indexed')

    if show:
        show_cmap_clev(cmap)

    return cmap


def get_cmap_and_clevs_by_element_name(element_name):
    path = None
    if element_name == "temp":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_temp_2m.txt")
    elif element_name == "rain_1h":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_1h.txt")
    elif element_name == "rain_1h_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_1h_error.txt")
    elif element_name == "rain_3h":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_3h.txt")
    elif element_name == "rain_3h_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rain_3h_error.txt")
    elif element_name == "rh":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rh.txt")
    elif element_name == "rh_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_rh_error.txt")
    elif element_name == "vis":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_vis.txt")
    elif element_name == "vis_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_vis_error.txt")
    elif element_name == "wind_speed":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_wind_speed.txt")
    elif element_name == "wind_speed_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_wind_speed_error.txt")
    elif element_name == "tcdc":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_tcdc.txt")
    elif element_name == "tcdc_error":
        path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_tcdc_error.txt")
    cmap,clevs = get_cmap_and_clevs_from_file(path)
    return cmap,clevs

def get_part_cmap_and_clevs(cmap_all,clev_all,vmax,vmin,cut_accurate = False):


    if cut_accurate:
        start_i = 0
        for i in range(len(clev_all) - 1):
            if vmin < clev_all[i + 1]:
                start_i = i
                break
        end_i = 0
        for i in range(len(clev_all) - 1):
            if vmax > clev_all[i]:
                end_i = i+2

    else:
        if len(clev_all) < 20:
            return cmap_all, clev_all
        start_i = 0
        for i in range(len(clev_all)-1):
            if vmin<clev_all[i+1]:
                start_i = i
                break
        end_i = 0
        for i in range(len(clev_all)-1):
            if vmax > clev_all[i]:
                end_i = i+2
        if end_i - start_i<=15:
            end_i = start_i+15


    clevs_part = clev_all[start_i:end_i]
    if hasattr(cmap_all,"colors"):
        cmap_colors = cmap_all.colors
        cmap_colors_part = cmap_colors[start_i:end_i]
        cmap_part = colors.ListedColormap(cmap_colors_part, 'indexed')
    else:
        cmap_part = cmap_all
    return cmap_part,clevs_part

def write_cmap_and_clevs(cmap,clevs,path):
    num = len(clevs)
    clev_cmap = np.zeros((num,4))
    clev_cmap[:,0] = clevs[:]
    cmap_data = np.array(cmap.colors)
    max_data = np.max(cmap_data)
    if max_data <=1:
        clev_cmap[:,1:] = cmap_data * 255
    else:
        clev_cmap[:, 1:] = cmap_data
    np.savetxt(path,clev_cmap,fmt = "%f")

def get_denser_cmap(cmap,multi_num):
    if hasattr(cmap,"colors"):
        colors0 = np.array(cmap.colors)
        colors_list = []
        num = len(colors0) - 1
        for i in range(num):
            for j in range(multi_num):
                color1 = (colors0[i,:]*(multi_num-j) + colors0[i+1,:] * j)/multi_num
                colors_list.append(color1.tolist())
        colors_list.append(colors0[-1])
        cmap_denser = colors.ListedColormap(colors_list, 'indexed')
        return cmap_denser
    else:
        return cmap


def show_cmap_clev(cmap,clev = None,extend = None,width = 10,height = 0.3,save_path = None,fontsize = 12):
    n_colors = len(cmap.colors)
    if clev is None:
        clev = np.arange(n_colors).tolist()

    # 创建一个空的 figure 和 axes
    fig, ax = plt.subplots(figsize=(width, height))

    # 创建一个空的 Normalize 对象，用于映射数据到颜色
    norm = BoundaryNorm(clev, ncolors=cmap.N)

    # 创建 colorbar 对象
    cbar = fig.colorbar(cm.ScalarMappable(norm=norm, cmap=cmap), cax=ax, orientation='horizontal', extend=extend)
    cbar.set_ticks(clev)

    cbar.ax.tick_params(labelsize=fontsize)    # 设置刻度标签字体大小
    if save_path is not None:
        plt.savefig(save_path,bbox_inches = "tight")
    else:
        plt.show()



# def show_cmap_clev(cmap,clev = None,save_path = None):
#     """
#     Show color map.
#     :param cmap: color map instance.
#     :return: None
#     """
#     n_colors = len(cmap.colors)
#     width = 8
#     heigh = 1
#     n_h = int(math.ceil(heigh * n_colors/width))
#     im = np.outer(np.ones(n_h), np.arange(n_colors))
#
#     fig, ax = plt.subplots(1, figsize=(width, heigh),
#                            subplot_kw=dict(xticks=[], yticks=[]))
#     ax.spines['bottom'].set_linewidth(1)
#     ax.spines['left'].set_linewidth(1)
#     ax.spines['top'].set_linewidth(1)
#     ax.spines['right'].set_linewidth(1)
#     if clev is not None:
#         max_tick = 10
#         step = int(math.ceil(n_colors/max_tick))
#         x = np.arange(0,n_colors,step).astype(np.int32)
#         #print(x)
#         ax.set_xticks(x)
#         labels = []
#         for i in range(x.size):
#             labels.append(round(clev[x[i]],6))
#         ax.set_xticklabels(labels)
#     ax.imshow(im, cmap=cmap)
#     if save_path is not None:
#         plt.savefig(save_path,bbox_inches='tight')


def get_color_list(legend_num):
    colors_list = []
    if legend_num<=10:
        colors = cm.get_cmap("tab10")
        for i in range(legend_num):
            colors_list.append(colors(i))
    elif legend_num <=20:
        colors = cm.get_cmap("tab20")
        for i in range(legend_num):
            colors_list.append(colors(i))
    else:
        colors = cm.get_cmap('gist_rainbow', 128)
        for i in range(legend_num):
            color_grade = i / legend_num
            colors_list.append(colors(color_grade))
    return colors_list


def get_cmap_and_clevs_by_name(cmap_name,vmin,vmax):
    path = pkg_resources.resource_filename('meteva', "resources/colormaps/color_"+cmap_name+".txt")
    if os.path.exists(path):
        cmap,clevs = get_cmap_and_clevs_from_file(path)
    elif cmap_name == "bias":
        cmap,clevs = cmap_clevs_bias(vmax)
    elif cmap_name == "error":
        cmap,clevs = cmap_clevs_error(vmin,vmax)
    elif cmap_name == "me":
        cmap,clevs = cmap_clevs_me(vmin,vmax)
    elif cmap_name == "me_bwr":
        cmap,clevs = cmap_clevs_me_bwr(vmin,vmax)
    elif cmap_name == "me_w0":
        cmap,clevs = cmap_clevs_me_w0(vmin,vmax)
    elif cmap_name == "ts":
        cmap,clevs= cmap_clevs_ts()
    elif cmap_name == "far":
        cmap,clevs = cmap_clevs_far()
    elif cmap_name == "far":
        cmap,clevs = cmap_clevs_far()
    elif cmap_name == "mode":
        cmap, clevs = cmap_clevs_mode(vmax)
    elif cmap_name == "temper_2m_error":
        cmap, clevs = cmap_clevs_temper_2m_error()
    elif cmap_name == "temper_error_br":
        cmap, clevs = cmap_clevs_temper_error_br(vmax)
    elif cmap_name =="environment":
        cmap,clevs = cmap_clevs_environment()
    elif cmap_name =="radar":
        cmap,clevs = cmap_clevs_radar()
    elif cmap_name =="mae":
        cmap, clevs = cmap_clevs_mae(vmax)
    elif cmap_name =="hour":
        cmap,clevs = hour()
    else:
        print("该配色方案名称不识别")
        return None,None
    return cmap,clevs


class cmaps:
    rain_1h = "rain_1h"
    rain_1h_error = "rain_1h_error"
    rain_3h = "rain_3h"
    rain_3h_error = "rain_3h_error"
    rain_24h = "rain_24h"
    rain_24h_error = "rain_24h_error"
    temp_2m = "temp_2m"
    temper_2m_error = "temper_2m_error"
    rh = "rh"
    rh_error = "rh_error"
    vis = "vis"
    vis_error = "vis_error"
    wind_speed = "wind_speed"
    wind_speed_error = "wind_speed_error"
    tcdc = "tcdc"
    tcdc_error  = "tcdc_error"
    bias = "bias"
    error = "error"
    ts = "ts"
    far = "far"
    mr = "mr"
    mode = "mode"
    me = "me"
    temper_error_br ="temper_error_br"
    environment = "environment"
    radar = "radar"
    me_bwr = "me_bwr"
    me_w0 = "me_w0"
    mae = "mae"
    hour = "hour"


def coordinate_cmap_to_clevs(cmap,clevs):
    if hasattr(cmap, "colors"):
        colors0 = np.array(cmap.colors)
        colors_list = []
        ncmap = len(colors0)
        nclev = len(clevs)
        if(ncmap == nclev + 1 or ncmap== nclev-1):
            return cmap,clevs
        if nclev <2:
            print("clevs' size must bigger than 1")
        for i in range(nclev):
            if nclev>1:
                j = i * (ncmap-1) /(nclev-1)
            else:
                j = 0
            j0 = int(j)
            j1 = min(j0 + 1,ncmap-1)
            dj = j - j0
            color1 = (colors0[j0, :] * (1 - dj) + colors0[j1, :] * dj)
            colors_list.append(color1.tolist())
        cmap_co = colors.ListedColormap(colors_list, 'indexed')
        return cmap_co,clevs
    else:
        print("cmap is not colormap")
        return



def def_cmap_clevs(cmap = "rainbow",clevs = None,vmin = None,vmax = None,cut_colorbar = True,extend = None):
    #  # 判断是meteva自定义的颜色类型，这从meteva资源文件或函数里生成cmap1 和clevs1
    clevs1 = None
    cmap1 = None

    if isinstance(cmap,str):
        cmap_class = cmaps()
        if hasattr(cmap_class, cmap):
            if vmin is None:
                vmin = np.min(np.array(clevs))
            if vmax is None:
                vmax = np.max(np.array(clevs))
            if vmax is not None and vmin == vmax:vmin = vmax - 1
            cmap,clevs1= get_cmap_and_clevs_by_name(cmap, vmin, vmax)
    if isinstance(cmap,list):
        if isinstance(cmap[0],list):
            if isinstance(clevs,list):
                if len(clevs)==len(cmap)+1:
                    cmap.append(cmap[-1])
            cmap_array= np.array(cmap)
            if np.max(cmap_array)>1:
                cmap_array =cmap_array / 256
            cmap_list = cmap_array.tolist()
        else:
            cmap_list = []
            for c1 in cmap:
                r1 = int(c1[1:3].upper(), 16)/256
                g1 = int(c1[3:5].upper(), 16)/256
                b1 = int(c1[5:7].upper(), 16)/256
                cmap_list.append([r1,g1,b1])
        cmap = colors.ListedColormap(cmap_list, 'indexed')


    #设置clevs2
    if clevs is None:
        if clevs1 is None:
            # 如果cmap字符不是meteva内集成的，则判断它是matplotlib集成的，则进一步根据最大最小值生成cmap，clevs
            if vmin is None or vmax is None:
                print("clev and vmin/vmax cann't be None at the same time while cmap is matplotlib strType cmap")
                return
            if vmax - vmin < 1e-10:
                vmax = vmin + 1.1
            dif = (vmax - vmin) / 10.0
            inte = math.pow(10, math.floor(math.log10(dif)));
            # 用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
            r = dif / inte
            if r < 3 and r >= 1.5:
                inte = inte * 2
            elif r < 4.5 and r >= 3:
                inte = inte * 4
            elif r < 5.5 and r >= 4.5:
                inte = inte * 5
            elif r < 7 and r >= 5.5:
                inte = inte * 6
            elif r >= 7:
                inte = inte * 8

            vmin = inte * (math.floor(vmin / inte))
            vmax = inte * (math.ceil(vmax / inte)+0.5)
            clevs2 = np.arange(vmin, vmax, inte)
        else:
            clevs2 = clevs1
    else:
        clevs2 = clevs


    # 设置cmap2
    if cmap1 is None:
        if isinstance(cmap,str):
            nclev = len(clevs2)
            if extend =="both":
                nclev += 1
            colors0 = cm.get_cmap(cmap, nclev)
            colors_list = []
            for i in range(nclev):
                colors_list.append(colors0(i))
            cmap2 = colors.ListedColormap(colors_list, 'indexed')
        else:
            cmap2 = cmap
    else:
        cmap2 = cmap1


    #将cmap2 和clev2 协调
    cmap3,clevs3 = coordinate_cmap_to_clevs(cmap2,clevs2)


    # 从cmap3 和cmap3中提取部分colorbar

    if vmin is not None and vmax is not None and cut_colorbar:
        cmap4,clevs4 = get_part_cmap_and_clevs(cmap3, clevs3, vmax, vmin)
    else:
        cmap4,clevs4  = cmap3,clevs3

    colors_list = cmap4.colors
    if extend == "both":

        if len(colors_list)  == len(clevs4) +1:
            cmap5 = colors.ListedColormap(colors_list[1:-1])
            cmap5.set_under(colors_list[0])
            cmap5.set_over(colors_list[-1])
        elif len(colors_list) == len(clevs4):
            cmap5 = colors.ListedColormap(colors_list[:-1])
            cmap5.set_under([1, 1, 1])
            cmap5.set_over(colors_list[-1])
        else:
            cmap5 = colors.ListedColormap(colors_list)
            cmap5.set_under([1, 1, 1])
            cmap5.set_over([0, 0, 0])

    elif extend == "min":
        cmap5 = colors.ListedColormap(colors_list[1:])
        cmap5.set_under(colors_list[0])
    elif extend == "max":
        cmap5 = colors.ListedColormap(colors_list[:-1])
        cmap5.set_over(colors_list[-1])
    else:
        cmap5 = cmap4

    return cmap5,clevs4


def merge_cmap_clevs(cmap0,clevs0,cmap1,clevs1):
    '''
    合并两个colorbar
    :param cmap0: 第0个colorbar的cmap
    :param clevs0:第0个colorbar的clevs
    :param cmap1:第1个colorbar的cmap
    :param clevs1:第1个colorbar的clevs
    :return:
    '''
    colors0 = np.array(cmap0.colors).tolist()
    colors1 = np.array(cmap1.colors).tolist()
    colors0.extend(colors1)
    cmap_m = colors.ListedColormap(colors0, 'indexed')
    clevs0.extend(clevs1)
    return cmap_m,clevs0


def cart2sph(x, y, z):
    XsqPlusYsq = x ** 2 + y ** 2
    r = math.sqrt(XsqPlusYsq + z ** 2)  # r
    lat = math.atan2(z, math.sqrt(XsqPlusYsq))  # theta
    lon = math.atan2(y, x)  # phi
    return r, lat, lon


def get_seprated_rgb_method1(num):
    '''
    通过再hsl球体空间种，寻找距离最远的num个点来，获得对比度最大的num个颜色
    :param num:  需要返回的颜色数目
    :return:  rgb列表
    '''
    loc0 = np.zeros((3, num + 1))
    loc0[:, 0] = [0, 0, 1]  # 固定第一个点是白色
    loc0[:, 1] = [0, 1, 0]  # 固定第二个点是红色
    loc0[0, 2:] = np.arange(1, num) / num  #初始化其他点的位置

    # 将每个颜色看作hsl空间种的一个同号带电粒子，根据每个受力调整其位置，逐步迭代方法，使它们距离更远。
    for i in range(20000):
        # 计算受力
        force = np.zeros((3, num))
        for j in range(1, num):
            other = np.zeros((3, num))
            other[:, 0:j + 1] = loc0[:, 0:j + 1]
            other[:, j + 1:] = loc0[:, j + 2:]
            this = loc0[:, j + 1]
            dxyz = this.T - other.T

            dis2 = np.sum(np.power(dxyz, 2), axis=1)
            force_j_m = dxyz.T / dis2.T
            force_j = np.sum(force_j_m, axis=1)
            force[:, j] = force_j[:]
        # 根据受力方向调整每个
        force_s = np.sqrt(np.sum(np.power(force, 2), axis=0))
        mean_f = np.mean(force_s)
        for j in range(1, num):
            if force_s[j] > mean_f:
                move = 0.009 * force[:, j] / force_s[j]
                loc_new = loc0[:, j + 1] + move
                dis0 = np.sqrt(np.sum(np.power(loc_new, 2)))
                if dis0 > 1:
                    loc_new /= dis0
                loc0[:, j + 1] = loc_new
    # 直角坐标转换成球坐标，再转换成hsl色彩模式
    loc0 = loc0.T.tolist()
    loc0.pop(0)
    loc1 = []
    for j in range(num):
        r, lat, lon = cart2sph(loc0[j][0], loc0[j][1], loc0[j][2])
        s = r
        h = lon / (2 * math.pi)
        l = 0.5 + lat / math.pi
        loc1.append([h, l, s])
    loc1 = np.array(loc1)
    loc1[:, 0] -= loc1[0, 0]
    loc2 = loc1[loc1[:, 0].argsort(), :]  # 按照色相排序
    loc3 = np.zeros(loc2.shape)
    index = np.where(loc2[:, 0] < 0)
    nm = len(index[0])
    loc3[num - nm:, :] = loc2[0:nm, :]
    loc3[:num - nm, :] = loc2[nm:, :]  # 将红色调整到第一个

    # 转换成rgb
    rgb_colors = []
    for j in range(num):
        rgb1 = colorsys.hls_to_rgb(loc3[j, 0], loc3[j, 1], loc3[j, 2])
        rgb_colors.append(rgb1)
    return rgb_colors


def get_seprated_rgb_method2(num):
    '''
    设置较大区分度的一组颜色
    :param num:  需要返回的颜色数目
    :return:  rgb列表
    '''
    rgb_colors = []
    step = 360.0 / num
    for i in range(num):
        h = i * step / 360  # 首先均匀的取不同的色相，保持色相维度的差异最大化
        i1 = i % 6
        if i1 <= 2:
            di = 0.5 - i1
        else:
            di = 0.5 + i1 - 4
        s = 0.75 + 0.25 * di / 1.5  # 通过一个折线波浪 设置不同的饱和度
        l = 0.5 + 0.25 * di / 1.5    # 通过一个折线波浪 设置不同的亮度
        rgb1 = colorsys.hls_to_rgb(h, l, s)
        rgb_colors.append(rgb1)

    return rgb_colors


def set_plot_color_dict_method0(member_list):
    if len(member_list) >20:
        print("tab20颜色方案只能支持member_list长度小于等于20的情况")
        return

    cm0 = cm.get_cmap("tab20")
    color_list0 = []
    for i in range(20):
        color_list0.append(cm0(i))
    plot_color_dict = {}
    for i in range(len(member_list)):
        plot_color_dict[member_list[i]] = color_list0[i]
    meteva.base.plot_color_dict = plot_color_dict
    return

def set_plot_color_dict_method1(member_list):
    color_list0 = get_seprated_rgb_method1(len(member_list))
    plot_color_dict = {}
    for i in range(len(member_list)):
        plot_color_dict[member_list[i]] = color_list0[i]
    meteva.base.plot_color_dict = plot_color_dict
    return

def set_plot_color_dict_method2(member_list):
    color_list0 = get_seprated_rgb_method2(len(member_list))
    plot_color_dict = {}
    for i in range(len(member_list)):
        plot_color_dict[member_list[i]] = color_list0[i]
    meteva.base.plot_color_dict = plot_color_dict
    return


def get_black_part(im_array):
    # 获取图形中的黑色部分，其中包含了colorbar的两条平行边框线
    # 没有黑色边框线包围的colorbar无法通过本算法识别
    r = im_array[:, :, 0]
    g = im_array[:, :, 1]
    b = im_array[:, :, 2]
    delta = np.abs(r - g) + np.abs(r - b) + np.abs(g - b)
    gray = r + g + b
    black = np.zeros(r.shape)
    black[delta < 100] = 1  # 容许有一定的噪音，不一定要纯黑
    black[gray > 100] = 0
    return black


def get_line_start_end_point(line):
    # 获取黑色线段的起止点，它容许因为噪音的缘故在line上出现少量的断点
    num = len(line)
    max_nstep = 0

    # 首先获得line中各段的起始坐标和长度
    start_list = []
    end_list = []

    # 记录每个黑色线段的起止点
    for k in range(1, num):
        if line[k - 1] == 0 and line[k] == 1:
            start_list.append(k)
        if line[k - 1] == 1 and line[k] == 0 and len(start_list) > 0:  # 如果线段一开始就是黑色，也就找不到起点，那终点也不记录
            end_list.append(k)

    # 如果线段最后是以黑色结尾，那就只有开始，没有结束，那最后一个开始点也不记录
    if len(start_list) > len(end_list):
        start_list = start_list[:-1]

    # 如果线段的终点和下一个起点很近，就把它们拼接起来
    start_list1 = [start_list[0]]
    end_list1 = []
    for i in range(1, len(start_list)):
        e0 = end_list[i - 1]
        s1 = start_list[i]
        if s1 - e0 > 4:  # 间隔2个点
            start_list1.append(start_list[i])
            end_list1.append(end_list[i - 1])
    end_list1.append(end_list[-1])

    # 保留长度大于 num * 0.05的线段
    se_list = []
    for i in range(len(start_list1)):
        if end_list1[i] - start_list1[i] > num * 0.05:
            se_list.append([start_list1[i], end_list1[i]])

    return se_list


def is_colorbar(array):
    nx = array.shape[0]
    ny = array.shape[1]
    mean = np.mean(array, axis=0)

    # j计算总亮度
    mean_sum = np.sum(mean, axis=1)
    mean_sum.sort()
    k = int(len(mean_sum) * 0.66)
    p66 = mean_sum[k]

    if p66 < 750:
        # 如果矩形区域内超过1/3的面积是不是白色
        mean = mean.reshape(1, ny, 3)
        delta = np.sum(np.abs(array - mean), axis=2)  # 在矩形区域内，垂直colorbar的方向的颜色变化幅度
        max_delta = np.max(delta)  # 变化幅度的最大值
        if max_delta < 30:
            # 如果变化幅度普遍很小，则说明是colorbar，理论上max_delta = 0,但有些图片可能不够清晰，有噪音存在
            return True
        else:
            return False
    else:
        # 如果矩形区域内超过2/3的面积是白色，则它应该不是colorbar
        return False


def get_cb_list(black, im3):
    nx = black.shape[0]
    ny = black.shape[1]
    thre = ny * 0.1
    # 查询横向长线段
    line_list = []
    for i in range(1, nx - 1):
        sumy = np.sum(black[i, :])
        if sumy > thre:
            line = black[i, :]
            se_list = get_line_start_end_point(line)
            for se in se_list:
                len1 = se[1] - se[0] + 1
                if np.sum(black[i - 1, se[0]:se[1]]) <= len1 / 2 or np.sum(black[i + 1, se[0]:se[1]]) <= len1 / 2:
                    line_list.append([i, se[0], i, se[1]])

    # 查询成对的横向线段
    nline = len(line_list)
    para_line_list = []
    for i in range(nline):
        line_i = line_list[i]
        for j in range(i + 1, nline):
            line_j = line_list[j]
            #             print(line_i)
            #             print(line_j)
            if np.abs(line_i[1] - line_j[1]) + np.abs(line_i[3] - line_j[3]) < 6:
                if line_j[0] - line_i[0] > 5:
                    para_line_list.append([line_i, line_j])
                break

    # 判断平行的线段是否构成长方形
    npara = len(para_line_list)
    re_list = []
    for i in range(npara):
        line0 = para_line_list[i][0]
        line1 = para_line_list[i][1]
        # 如果高比宽还大，就不考虑它是colorbar
        if line1[0] - line0[0] < line0[3] - line0[1]:
            re_list.append(para_line_list[i])

    nr = len(re_list)
    cb_list = []
    # 判断平行线内是否为colorbar
    for i in range(nr):
        line0 = re_list[i][0]
        line1 = re_list[i][1]
        xs = line0[0] + 2
        ys = max(line0[1], line1[1]) + 1
        xe = line1[0] - 2
        ye = min(line0[3], line1[3]) - 1

        array = im3[xs:xe, ys:ye, :]
        is_c = is_colorbar(array)
        if is_c:
            cb_list.append([xs, xe, ys, ye])
    return cb_list


def get_cmap_from_line(line_array):
    # 从扣除的colorbar的中间一条矩形图像平均得到的一条线中提取颜色等级

    len1 = line_array.shape[0]  # 线的长度
    delta = np.zeros(len1)
    for i in range(1, len1):
        delta[i] = np.sum(np.abs(line_array[i, :-1] - line_array[i - 1, :-1]))  # 计算线段上相邻相似点在不同颜色分量（rgb）的变化

    # 判断colorbar是突变的还是渐变的
    mean_delta = np.mean(delta)
    delta_index = np.zeros(len1)
    delta_index[0] = 1
    delta_index[delta > mean_delta] = 1  # 大于平均值的记为1，否则记为0
    color_list = []
    if np.mean(delta_index) < 0.1:
        # 如果只有少量的点的变化幅度大于平均值，说明颜色是突变形式的
        for i in range(len1 - 5):
            if delta_index[i - 1] == 1 and np.sum(delta_index[i:i + 5]) == 0:
                # 取颜色突变，且之后维持5像素不变的位置的颜色
                color_list.append(line_array[i, :])
    else:
        # 均匀选取颜色
        step = int(math.ceil(len1 / 100))
        for i in range(2, len1 - 2, step):
            color_list.append(line_array[i, :])
    cmap = colors.ListedColormap(color_list, 'indexed')

    return cmap


def get_cmap_from_im(im):
    im255 = im[:, :, :3] * 255
    black = get_black_part(im255)
    cb_list = get_cb_list(black, im255)
    nt = len(cb_list)
    cmap_list = []
    for i in range(nt):
        cb = cb_list[i]

        w = cb[1] - cb[0]  # colorbar的宽度
        mid = int((cb[1] + cb[0]) / 2)  # colorbar中线的x坐标

        si = int(mid - w / 4)  # 取colorbar的中间1/2 的一条进入颜色提取环节
        ei = int(mid + w / 4) + 1

        sj = cb[2]  # 默认colorbar的长度和两条平行黑线的长度一致
        ej = cb[3]
        ###################################################
        # 为colorbar两段可能出现的三角形扩展 y坐标的起止点
        # 扩展起点
        for q in range(cb[2], -1, -1):
            if black[mid, q] == 0:
                break
        if cb[2] - q > 20:
            p = q
        else:
            for p in range(q, -1, -1):
                if black[mid, p] == 1:
                    break

        if cb[2] - p > 5 and p > 0:
            sum_b = 0
            sum_w = 0
            for k in range(p, cb[2]):
                h = 0.5 * (k - p) * w / (cb[2] - p)
                hu = int(h + mid)
                sum_b += np.max(black[hu - 2:hu + 5, k])
                sum_w += np.max(black[hu + 5:hu + 6, k])
            if sum_b / (cb[2] - p) > 0.95 and sum_w / (cb[2] - p) < 0.05:
                sj = p

                # 扩展终点
        for q in range(cb[3], black.shape[1]):
            if black[mid, q] == 0:
                break
        if q - cb[3] > 20:
            p = q
        else:
            for p in range(q, black.shape[1]):
                if black[mid, p] == 1:
                    break

        if p - cb[3] > 5 and p < black.shape[1] - 1:
            sum_b = 0
            sum_w = 0
            for k in range(p, cb[3], -1):
                h = 0.5 * (p - k) * w / (p - cb[3])
                hu = int(h + mid)
                sum_b += np.max(black[hu - 2:hu + 5, k])
                sum_w += np.max(black[hu + 5:hu + 6, k])
            if sum_b / (p - cb[3]) > 0.95 and sum_w / (p - cb[3]) < 0.05:
                ej = p

                # 为colorbar两段可能出现的三角形扩展 y坐标的起止点
        ################################################3
        line = np.mean(im[si:ei, sj:ej, :], axis=0)  # 提取一条图形数据，并取平均，得到一条线
        cmap = get_cmap_from_line(line)  # 从图形数据中识别颜色等级
        cmap_list.append(cmap)

    return cmap_list


def get_cmap_from_picture(path, show=False):
    im = image.imread(path)
    cmap_list = get_cmap_from_im(im)  # 获取横向colorbar

    # 将图片顺时针旋转90度，将图片中的中旬colorbar转换成横向，再识别
    im_t = np.zeros((im.shape[1], im.shape[0], 4))
    im_t[:, :, 0] = im[::-1, :, 0].T
    im_t[:, :, 1] = im[::-1, :, 1].T
    im_t[:, :, 2] = im[::-1, :, 2].T
    im_t[:, :, 3] = im[::-1, :, 3].T
    cmap_list_t = get_cmap_from_im(im_t)

    cmap_list.extend(cmap_list_t)  # 将两种可能得colorbar都归集在一起
    if show:
        for cmap in cmap_list:
            meteva.base.tool.color_tools.show_cmap_clev(cmap, clev=np.arange(len(cmap.colors)))
    return cmap_list


def creat_cmap_from_rgb(rgb_list_list,extend = None):
    if isinstance(rgb_list_list[0],list):
        colors_list = np.array(rgb_list_list)/256
    else:
        colors_list =rgb_list_list
    if extend == 'min':
        cmap = colors.ListedColormap(colors_list[1:])
        cmap.set_under(colors_list[0])
    elif extend == 'max':
        cmap = colors.ListedColormap(colors_list[:-1])
        cmap.set_over(colors_list[-1])
    elif extend == 'both':
        cmap = colors.ListedColormap(colors_list[1:-1])
        cmap.set_under(colors_list[0])
        cmap.set_over(colors_list[-1])
    else:
        cmap = colors.ListedColormap(colors_list)

    return cmap



import meteva
import numpy as np
from scipy.spatial import cKDTree
from scipy.interpolate import LinearNDInterpolator
import copy
import pandas as pd
import math


def reset_global_griddata(grd):
    '''
    考虑全球网格展成平面时，有一条经度带不能完全覆盖，为此扩展一列，以覆盖全球
    :param grd:
    :return:
    '''
    grid = meteva.base.get_grid_of_data(grd)
    if  grid.glon[1] + grid.glon[2] - grid.glon[0] -360 > -0.001:
        glon1 = [grid.glon[0],grid.glon[1]+grid.glon[2],grid.glon[2]]
        grid1 = meteva.base.grid(glon1,grid.glat,grid.gtime,grid.dtimes,grid.levels,grid.members)
        grd1 = meteva.base.grid_data(grid1)
        grd1.values[:,:,:,:,:,:-1] = grd.values[:,:,:,:,:,:]
        grd1.values[:, :, :, :, :, -1] = grd.values[:, :, :, :, :, 0]
        return  grd1
    else:
        return grd

def reset_lon_range(grd,sta):
    '''
    经度的取值范围有-180 —— 180 和 0-360 两种方式， 此函数将grd 的取值范围向station 统一
    :param grd:
    :param station:
    :return:
    '''
    grid = meteva.base.get_grid_of_data(grd)
    slon0 = np.min(sta.loc[:, "lon"].values)
    if slon0 < 0 and grid.glon[1] > 180:
        lons0 = grd["lon"].values
        nlon = lons0.size
        index = np.argmax(lons0>180)
        lons1 = np.zeros(nlon)
        lons1[0:nlon - index] = lons0[index:nlon] - 360
        lons1[nlon - index:] = lons0[0:index]
        glon1 = [lons1[0],lons1[-1],lons1[1] - lons1[0]]
        grid1 = meteva.base.grid(glon1,grid.glat,grid.gtime,grid.dtimes,grid.levels,grid.members)
        grd1 = meteva.base.grid_data(grid1)
        grd1.values[:,:,:,:,:,0:nlon - index] = grd.values[:,:,:,:,:,index:nlon]
        grd1.values[:, :, :, :, :, nlon - index:] = grd.values[:, :, :, :, :, 0:index]
        return grd1
    else:
        elon0 = np.max(sta.loc[:, "lon"].values)
        if elon0 >180 and grid.glon[0] <0:
            lons0 = grd["lon"].values
            nlon = lons0.size
            index = np.argmax(lons0 >=0)
            lons1 = np.zeros(nlon)
            lons1[0:nlon - index] = lons0[index:nlon]
            lons1[nlon - index:] = lons0[0:index] + 360
            grid1 = meteva.base.grid(lons1, grid.glat, grid.gtime, grid.dtimes, grid.levels, grid.members)
            grd1 = meteva.base.grid_data(grid1)
            grd1.values[:, :, :, :, :, 0:nlon - index] = grd.values[:, :, :, :, :, index:nlon]
            grd1.values[:, :, :, :, :, nlon - index:] = grd.values[:, :, :, :, :, 0:index]
            return grd1
        else:
            return grd


#格点到站点的插值
def interp_gs_nearest(grd,sta,used_coords = "xy"):
    '''
    :param grd:
    :param sta:
    :return:
    '''
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)

    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        if len(sta1.index)==0:return None
        ig = np.round((sta1.loc[:,'lon'].values - grid.slon) / grid.dlon).astype(dtype = 'int16')
        jg = np.round((sta1.loc[:,'lat'].values - grid.slat) / grid.dlat).astype(dtype = 'int16')
        sta_list = []
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,["time","id","lon","lat"]]
                    sta.loc[:,'time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        dat_sta= dat[jg,ig]
                        sta.loc[:,members[m]] = dat_sta
                    sta_list.append(sta)
        #sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = pd.concat(sta_list,axis = 0)
        sta_all = sta_all.reindex(columns=column_list2)
        sta_all.attrs = copy.deepcopy(grd.attrs)
        return sta_all


#格点到站点的插值,线性
def interp_gs_linear(grd,sta,used_coords = "xy"):

    grd = reset_lon_range(grd,sta)
    grd = reset_global_griddata(grd)

    #print("**0")
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)
    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        ig = ((sta1['lon'].values - grid.slon) // grid.dlon).astype(dtype = 'int16')
        jg = ((sta1['lat'].values - grid.slat) // grid.dlat).astype(dtype = 'int16')
        dx = (sta1['lon'].values - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'].values - grid.slat) / grid.dlat - jg
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1-dx) * dy
        c11 = dx * dy
        ig1 = np.minimum(ig + 1, grid.nlon - 1)
        jg1 = np.minimum(jg + 1, grid.nlat - 1)
        sta_list = []
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,[ "id", "lon", "lat"]]
                    sta['time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        dat_sta= c00 * dat[jg,ig] + c01 * dat[jg,ig1] + c10 * dat[jg1,ig] + c11 * dat[jg1,ig1]
                        sta.loc[:,members[m]] = dat_sta
                    sta_list.append(sta)
        #sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = pd.concat(sta_list, axis=0)
        sta_all = sta_all.reindex(columns=column_list2)
        sta_all.attrs = copy.deepcopy(grd.attrs)
        return sta_all

    elif used_coords == "xyz":
        sta1 = meteva.base.sele.in_grid_xyz(sta, grid)
        lev_s = sta1["level"].values[:]
        lev_g = np.array(grid.levels)
        kg = meteva.base.tool.math_tools.get_index(lev_s,grid.levels)
        kg1 = np.minimum(kg + 1, len(grid.levels) - 1)
        ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype='int16')
        jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype='int16')
        dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
        dz = (lev_s - lev_g[kg])/(lev_g[kg1] - lev_g[kg] + 1e-30)
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1 - dx) * dy
        c11 = dx * dy
        ig1 = np.minimum(ig + 1, grid.nlon - 1)
        jg1 = np.minimum(jg + 1, grid.nlat - 1)
        sta_all = None
        for j in range(len(times)):
            for k in range(len(dtimes)):
                sta = sta1.loc[:,["level","id","lon","lat"]]
                sta.loc[:,'time'] = times[j]
                sta.loc[:,'dtime'] = dtimes[k]
                for m in range(len(members)):
                    dat = grd.values[m,:,j,k,:,:]
                    dat_sta= (c00 * dat[kg,jg,ig] + c01 * dat[kg,jg,ig1] + c10 * dat[kg,jg1,ig] + c11 * dat[kg,jg1,ig1]) * (1-dz)
                    dat_sta += (c00 * dat[kg1, jg, ig] + c01 * dat[kg1, jg, ig1] + c10 * dat[kg1, jg1, ig] + c11 * dat[
                        kg1, jg1, ig1]) *  dz
                    sta.loc[:,members[m]] = dat_sta
                sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        sta_all.attrs = copy.deepcopy(grd.attrs)
        return sta_all

    elif used_coords =="xydt":
        if(len(grid.dtimes) == 1):
            print("dtime维度size = 1,无法开展dtime维度插值")
            return
        '''
        还有问题
        '''
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        sta1 = meteva.base.sele.between_dtime_range(sta1,grid.dtimes[0],grid.dtimes[-1])
        dtime_g = np.array(grid.dtimes)
        lon_g = grd["lon"].values
        lat_g = grd["lat"].values

        coords_g = np.zeros((dtime_g.size, lat_g.size, lat_g.size, 3))
        coords_g[..., 0] = dtime_g.reshape((dtime_g.size,1, 1))
        coords_g[..., 1] = lat_g.reshape((1, lat_g.size, 1))
        coords_g[..., 2] = lon_g.reshape((1,1, lon_g.size))
        coords_g = coords_g.reshape((dtime_g.size * lat_g.size * lat_g.size, 3))

        coords_s = np.zeros((len(sta1.index), 3))
        coords_s[:, 0] = sta1["dtime"].values
        coords_s[:, 1] = sta1["lat"].values
        coords_s[:, 2] = sta1["lon"].values
        sta_all = None
        for j in range(len(times)):
            for k in range(len(levels)):
                sta = sta1.loc[:,["dtime","id","lon","lat"]]
                sta.loc[:,'time'] = times[j]
                sta.loc[:,'level'] = levels[k]
                for m in range(len(members)):
                    dat_g = grd.values[m,k,j,:,:,:]
                    interpolator = LinearNDInterpolator(coords_g, dat_g.reshape((dat_g.size)))
                    dat_s = interpolator(coords_s)
                    sta.loc[:, members[m]] = dat_s
                sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        sta_all.attrs = copy.deepcopy(grd.attrs)
        return sta_all


#格点到站点的插值，三次
def interp_gs_cubic(grd,sta,used_coords = "xy"):
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)
    sta_all = None
    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype='int16')
        jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype='int16')
        dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,["id","lon","lat"]]
                    sta.loc[:,'time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        sum = np.zeros(len(sta.index))
                        for p in range(-1, 3, 1):
                            iip = np.minimum(np.maximum(ig + p, 0), grid.nlon - 1)
                            fdx = cubic_f(p, dx)
                            for q in range(-1, 3, 1):
                                jjq = np.minimum(np.maximum(jg + q, 0), grid.nlat - 1)
                                fdy = cubic_f(q, dy)
                                fdxy = fdx * fdy
                                sum[:] += fdxy * dat[jjq, iip]
                        sta.loc[:, members[m]] =sum
                    sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        sta_all.attrs = copy.deepcopy(grd.attrs)
        return sta_all


    grid = meteva.base.basicdata.get_grid_of_data(grd)
    sta1 = meteva.base.sele.in_grid_xy(sta, grid)
    dat0 = grd.values
    dat = np.squeeze(dat0)
    ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype = 'int16')
    jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype = 'int16')
    dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
    dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
    data_name = meteva.base.get_stadata_names(sta1)[0]
    for p in range(-1,3,1):
        iip = np.minimum(np.maximum(ig+p,0),grid.nlon-1)
        fdx = cubic_f(p, dx)
        for q in range(-1,3,1):
            jjq = np.minimum(np.maximum(jg+q,0),grid.nlat-1)
            fdy = cubic_f(q,dy)
            fdxy = fdx * fdy
            sta1[data_name] +=  fdxy * dat[jjq,iip]
    sta1['time'] = grid.stime
    sta1['dtime'] = grid.dtimes[0]
    sta1['level'] = grid.levels[0]
    meteva.base.basicdata.set_stadata_names(sta1,grid.members)
    sta_all.attrs = copy.deepcopy(grd.attrs)
    return sta1


def interp_sg_idw(sta0, grid, background=None, effectR=1000, nearNum=8,decrease = 2):

    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        dtime = sta.loc[index0, 'dtime'].astype(int)
        level = sta.loc[index0, 'level'].astype(int)
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [dtime],
                                                           [level], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        if nearNum > len(sta.index):
            nearNum = len(sta.index)
        d, inds = tree.query(xyz_grid, k=nearNum)
        if nearNum >1:
            d += 1e-6
            w = 1.0 / d ** decrease
            input_dat = sta.values[:,-1]
            dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
            bg = meteva.base.basicdata.grid_data(grid2)
            if (background is not None):
                bg = interp_gg_linear(background, grid2)
            bg_dat = bg.values.flatten()
            dat = np.where(d[:, 0] > effectR, bg_dat, dat)
        else:
            input_dat = sta.iloc[:,-1].values
            dat = input_dat[inds]
            bg = meteva.base.basicdata.grid_data(grid2)
            if (background is not None):
                bg = interp_gg_linear(background, grid2)
            bg_dat = bg.values.flatten()
            dat = np.where(d[:] > effectR, bg_dat, dat)
        dat = dat.astype(np.float32)
        grd = meteva.base.basicdata.grid_data(grid2, dat)
        grd.name = data_name[0]
        grd_list.append(grd)

    grd_all = meteva.base.concat(grd_list)
    grd_all.attrs = copy.deepcopy(sta0.attrs)
    return grd_all


def interp_sg_idw_delta(sta0, grid,  halfR=1000, nearNum=8,decrease = 2):
    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [sta.loc[index0, 'dtime']],
                                                           [sta.loc[index0, 'level']], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        if nearNum > len(sta.index):
            nearNum = len(sta.index)
        d, inds = tree.query(xyz_grid, k=nearNum)
        if nearNum >1:
            d += 1e-6
            w1 = 1.0 / d ** decrease
            w2 = np.exp(-(d/halfR)**2)
            input_dat = sta.values[:,-1]
            dat = np.sum(w1 * w2 * input_dat[inds], axis=1) / np.sum(w1, axis=1)
        else:
            input_dat = sta0.iloc[:,-1].values
            w2 = np.exp(-d/halfR)
            dat = w2 * input_dat[inds]
        dat = dat.astype(np.float32)
        grd = meteva.base.basicdata.grid_data(grid2, dat)
        grd.name = data_name[0]
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    grd_all.attrs = copy.deepcopy(sta0.attrs)
    return grd_all


def interp_sg_cressman(sta0, grid, r_list,background=None , nearNum=100):
    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [sta.loc[index0, 'dtime']],
                                                           [sta.loc[index0, 'level']], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        nsta = len(sta.index)
        if nearNum > nsta:
            nearNum = nsta
        d, inds = tree.query(xyz_grid, k=nearNum)
        d += 1e-6
        bg = meteva.base.basicdata.grid_data(grid2)
        if (background is not None):
            bg = interp_gg_linear(background, grid2)

        bg_dat = bg.values.flatten()
        input_dat = sta.values[:, -1]

        d2 = d ** 2
        for R in r_list:
            index_in = np.where(d[:,0] < R)[0]
            inds_in = inds[index_in,:]
            r2 = R ** 2
            d2_in = d2[index_in,:]
            w = (r2 -  d2_in)/(r2 + d2_in)
            w[w <0] = 0
            dat = np.sum(w * input_dat[inds_in], axis=1) / np.sum(w, axis=1)
            bg_dat[index_in] = dat[:]
        grd = meteva.base.basicdata.grid_data(grid2, bg_dat)
        grd.name = data_name[0]
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    grd_all.attrs = copy.deepcopy(sta0.attrs)
    return grd_all


def interp_ss_idw(sta0, station, effectR=1000, nearNum=8,decrease = 2,defalut_value =0):
    '''
    :param sta0: 包含原始数据的站点数据
    :param station: 插值后的目标站点
    :param effectR: 反距离插值最大半径
    :param nearNum: 插值所选用的最近点个数
    :return:
    '''
    sta = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta1 = station.copy()
    xyz_sta0 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,sta['lat'].values,R=meteva.base.basicdata.const.ER)
    xyz_sta1 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta1['lon'].values,sta1['lat'].values,R=meteva.base.basicdata.const.ER)
    tree = cKDTree(xyz_sta0)
    d, inds = tree.query(xyz_sta1, k=nearNum)
    if nearNum >1:
        d += 1e-6
        w = 1.0 / d ** decrease
        input_dat = sta0.iloc[:,-1].values
        dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
        dat[:] = np.where(d[:,0] > effectR,defalut_value,dat[:])
        sta1.iloc[:,-1] = dat[:]
    else:
        input_dat = sta0.iloc[:,-1].values
        dat = input_dat[inds]
        dat[:] = np.where(d[:] > effectR,defalut_value,dat[:])
        sta1.iloc[:,-1] = dat[:]
    sta1.attrs = copy.deepcopy(sta0.attrs)
    return sta1


def interp_gg_nearest(grd, grid,used_coords = "xy",outer_value = None):
    '''
    格点到格点插值
    :param grd:左边的网格数据信息
    :param grid :右边的网格数据信息
    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
    否则采用grid里的值，默认为：left
    :return:双线性插值之后的结果
    '''
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    iscycle = (grid0.dlon * grid0.nlon >= 360)
    if used_coords == "xy":
        is_out = False
        if not iscycle:
            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True

        if is_out:
            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        else:
            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        grd_new = meteva.base.grid_data(grid_new)
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值
                        dat = grd.values[m,i,j,k,:,:]
                        #插值处理
                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
                        ig = np.around(x[:]).astype(dtype='int16')
                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
                        jg = np.around(y[:]).astype(dtype='int16')
                        ii, jj = np.meshgrid(ig, jg)
                        dat2 = dat[jj, ii]
                        grd_new.values[m,i,j,k,:,:] = dat2
        if is_out:
            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
    grd_new.attrs = copy.deepcopy(grd.attrs)
    return grd_new


def interp_gg_linear(grd, grid,used_coords = "xy",outer_value = None):
    '''
    格点到格点插值
    :param grd:左边的网格数据信息
    :param grid :右边的网格数据信息
    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
    否则采用grid里的值，默认为：left
    :return:双线性插值之后的结果
    '''
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    icycle = int(360/grid0.dlon)
    iscycle = (grid0.dlon * grid0.nlon >= 360)
    if used_coords == "xy":
        is_out = False
        if not iscycle:
            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True
        else:
            if (grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True

        if is_out:
            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        else:
            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        grd_new = meteva.base.grid_data(grid_new)
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值
                        dat = grd.values[m,i,j,k,:,:]
                        #插值处理
                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
                        ig = x[:].astype(dtype='int16')
                        dx = x - ig
                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
                        jg = y[:].astype(dtype='int16')
                        dy = y - jg
                        ii, jj = np.meshgrid(ig, jg)
                        if iscycle:
                            ii1 = ii+1
                            ii = ii%icycle
                            ii1 = ii1%icycle
                        else:
                            ii1 = np.minimum(ii + 1, grid0.nlon - 1)
                        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
                        ddx, ddy = np.meshgrid(dx, dy)
                        c00 = (1 - ddx) * (1 - ddy)
                        c01 = ddx * (1 - ddy)
                        c10 = (1 - ddx) * ddy
                        c11 = ddx * ddy
                        dat2 = (c00 *dat[jj, ii] + c10 * dat[jj1, ii] + c01 * dat[jj, ii1] + c11 * dat[jj1, ii1])
                        grd_new.values[m,i,j,k,:,:] = dat2
        if is_out:
            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
    grd_new.attrs = copy.deepcopy(grd.attrs)
    return grd_new


def interp_gg_cubic(grd, grid,used_coords = "xy",outer_value = None):
    '''
    格点到格点插值
    :param grd:左边的网格数据信息
    :param grid :右边的网格数据信息
    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
    否则采用grid里的值，默认为：left
    :return:双线性插值之后的结果
    '''
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    icycle = int(360/grid0.dlon)
    iscycle = (grid0.dlon * grid0.nlon >= 360)
    if used_coords == "xy":
        is_out = False
        if not iscycle:
            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True

        if is_out:
            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        else:
            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        grd_new = meteva.base.grid_data(grid_new)
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值


                        dat = grd.values[m,i,j,k,:,:]
                        #插值处理
                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
                        ig = x[:].astype(dtype='int16')
                        dx = x - ig
                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
                        jg = y[:].astype(dtype='int16')
                        dy = y - jg
                        ii, jj = np.meshgrid(ig, jg)
                        if iscycle:
                            ii1 = ii+1
                            ii = ii%icycle
                            ii1 = ii1%icycle
                        else:
                            ii1 = np.minimum(ii + 1, grid0.nlon - 1)
                        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
                        ddx, ddy = np.meshgrid(dx, dy)

                        for p in range(-1, 3, 1):
                            iip = np.minimum(np.maximum(ii + p, 0), grid0.nlon - 1)
                            fdx = cubic_f(p, dx)
                            for q in range(-1, 3, 1):
                                jjq = np.minimum(np.maximum(jj + q, 0), grid0.nlat - 1)
                                fdy = cubic_f(q, dy)
                                fdxx, fdyy = np.meshgrid(fdx, fdy)
                                fdxy = fdxx * fdyy
                                grd_new.values[m,i,j,k,:,:] += fdxy * dat[jjq, iip]

        if is_out:
            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
    grd_new.attrs = copy.deepcopy(grd.attrs)
    return grd_new


def interp_xg_linear(dataArray,grid,used_coords = "xy"):
    meteva.base.reset(dataArray)
    levels = dataArray["level"].values
    times = dataArray["time"].values
    dtimes = dataArray["dtime"].values
    members = dataArray["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(dataArray)
    grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
    grd_new = meteva.base.grid_data(grid_new)


    if used_coords == "xy":

        # 插值处理
        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
        ig = x[:].astype(dtype='int16')
        dx = x - ig
        y = np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat
        nlat = len(y)
        jg = np.zeros(nlat).astype(dtype='int16')
        dy = np.zeros(nlat)
        lats_old = dataArray.lat.values
        for ny in range(nlat):
            js = 0
            for j in range(js,nlat-1):
                if y[ny] >= lats_old[j] and y[ny] <= lats_old[j+1]:
                    jg[ny] = j
                    dy[ny] = (y[ny] -  lats_old[j])/(lats_old[j+1] - lats_old[j])
                    js = j
                    break


        #print(jg)
        ii, jj = np.meshgrid(ig, jg)
        ii1 = np.minimum(ii + 1, grid0.nlon - 1)
        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
        ddx, ddy = np.meshgrid(dx, dy)
        c00 = (1 - ddx) * (1 - ddy)
        c01 = ddx * (1 - ddy)
        c10 = (1 - ddx) * ddy
        c11 = ddx * ddy


        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值
                        dat = dataArray.values[m, i, j, k, :, :]
                        dat2 = (c00 * dat[jj, ii] + c10 * dat[jj1, ii] + c01 * dat[jj, ii1] + c11 * dat[jj1, ii1])
                        grd_new.values[m, i, j, k, :, :] = dat2

    return grd_new


def cubic_f(n, dx):
    if (n == -1):
        return -dx * (dx - 1) * (dx - 2) / 6
    elif (n == 0):
        return (dx + 1) * (dx - 1) * (dx - 2) / 2
    elif (n == 1):
        return -(dx + 1) * dx * (dx - 2) / 2
    else:
        return (dx + 1) * dx * (dx - 1) / 6


def interp_zlevel_to_plevel_linear(pressure,element_interp,level_list,with_nan = False):
    '''
    将模式面的数据插值到等压面
    :param pressure: 模式面的气压数据，griddata格式，在level维度是包含多层的
    :param element_interp:  需要插值到等压面的要素数据,所有维度的坐标必须和pressure完全相同。
    :param level_list:  需要插值到的等压面层次，取值是气压值的列表，气压的单位需和 pressure变量中气压的单位完全一致。
    :param with_nan: 如果某个等压面层的某个区域或全部低于所有的模式面的气压或高于所有模式面的气压时，是否以nan的方式返回
    :return:  网格数据，在level维度坐标和 level_list一致，在member，time,dtime,lat,lon维度和pressure完全一致。
    '''

    if not isinstance(level_list,list) and not isinstance(level_list,np.ndarray):
        level_list = [level_list]

    A = pressure.values
    B = element_interp.values
    member_dim = np.arange(A.shape[0])[:,None,None,None,None,None]
    time_dim = np.arange(A.shape[2])[:,None,None,None]
    dtime_dim = np.arange(A.shape[3])[:,None,None]
    lat_dim = np.arange(A.shape[4])[:,None]
    lon_dim = np.arange(A.shape[5])[:]

    grid0 = meteva.base.get_grid_of_data(pressure)
    grid1 = meteva.base.grid(grid0.glon,grid0.glat,member_list=grid0.members,dtime_list=grid0.dtimes,
                    gtime=grid0.gtime,level_list=level_list)

    grd_inter = meteva.base.grid_data(grid1)

    for k in range(len(level_list)):
        level = level_list[k]
        mask = A < level
        index_smaller = np.argmax(mask,axis=1)
        index_bigger = index_smaller - 1
        p_smaller = A[member_dim,index_smaller,time_dim,dtime_dim,lat_dim,lon_dim]
        p_bigger = A[member_dim,index_bigger,time_dim,dtime_dim,lat_dim,lon_dim]

        element_smaller = B[member_dim, index_smaller, time_dim, dtime_dim, lat_dim, lon_dim]
        element_bigger = B[member_dim, index_bigger, time_dim, dtime_dim, lat_dim, lon_dim]
        rate = (p_bigger - level) /(p_bigger - p_smaller+1e-30)
        value = rate * element_smaller + (1-rate) * element_bigger
        if with_nan:
            index = np.where(index_smaller==0)
            value[:,:,:,:,index[3],index[4]] = np.nan

        grd_inter.values[:,k,:,:,:,:] = value

    return grd_inter



import copy
import math
import pathlib
import meteva
import datetime
import pandas as pd
import os
import numpy as np
import  time


def read_ob_wind(path):
    speed = meteva.base.read_stadata_from_micaps1_2_8(path,column=meteva.base.m2_element_column.风速)
    angle = meteva.base.read_stadata_from_micaps1_2_8(path,column=meteva.base.m2_element_column.风向)
    if speed is not None and angle is not None:
        speed = meteva.base.sele_by_para(speed,value=[0,100])
        wind = meteva.base.speed_angle_to_wind(speed,angle)
        return wind
    else:
        return None


para_example= {
    "base_on": "foTime",
    "mid_method": meteva.method.na_uv,  # 统计检验的总量计算函数，
    "grade_list": None,  # 二分类检验所需的等级参数
    "compare": None,  # 二分类检验所需的对比方式参数
    "begin_time":datetime.datetime(2024,9,1,8),
    "end_time":datetime.datetime(2024,9,3,20),
    "time_type":"BT",  # 最终检验结果呈现时，采用北京时还是世界时，UT代表世界时，BT代表北京时
    "time_step": 12,  # 起报时间间隔
    "middle_result_path": r"H:\test_data\output\mps\na.h5",  #检验中间量数据的存储路径
    "station_file":meteva.base.station_国家站,  #检验站点表的存储路径
    "defalut_value":meteva.base.IV,
    "interp": meteva.base.interp_gs_nearest,
    "recover":False,    #False表示保留已经收集好的中间量，补齐未收集的数据，True表示删除已有的统计结果文件，重新开始收集
    "cpu":6,
    "ob_data":{
        "dir_ob": r"\\10.20.22.27\rc\WIND\w2mi0\YYYYMMDDHH.000", # 实况场数据路径
        "hour":[8,20,12],
        "read_method": read_ob_wind,    #读取数据的函数
        "read_para": {},                          #读取数据的函数参数
        "reasonable_value": None,
        "operation":None,  #实况数据读取后处理函数
        "operation_para": {},  #实况数据读取后处理参数
        "time_type": "BT",  # 数据文件中的时间类型，UT代表世界时
    },
    "fo_data":{
        "ECMWF": {
            "dir_fo": r"S:\data\grid\ECMWF_HR\WIND_10M\YYYYMMDD\YYMMDDHH.TTT.nc", # 数据路径
            "hour": [8, 20, 12],
            "dtime":[12,24,12],                #检验的时效
            "read_method": meteva.base.io.read_griddata_from_nc,  #读取数据的函数
            "read_para": {},  #读取数据的函数参数
            "reasonable_value": None,
            "operation": None,  #预报数据读取后处理函数
            "operation_para": {},   #预报数据读取后处理参数
            "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
            "move_fo_time": 12   #是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
        },

        "SCMOC": {
            "dir_fo": r"S:\data\grid\NWFD_SCMOC\WIND\10M_ABOVE_GROUND\YYYYMMDD\YYMMDDHH.TTT.nc",
            "hour": [8, 20, 12],
            "dtime":[12,24,12],
            "read_method": meteva.base.io.read_griddata_from_nc,
            "read_para": {},
            "reasonable_value": None,
            "operation": None,
            "operation_para": {},
            "time_type": "BT",
            "move_fo_time": 0
        },
    },
    "output_dir":None  #观测站点合并数据的输出路径，设置为None时不输出收集数据的中间结果
}

def task_of_one_p_surface(para,time_fo):
    para_one = copy.deepcopy(para)
    para_one["begin_time"] = time_fo
    para_one["end_time"] = time_fo
    middle_result_dir_sep = os.path.split(para_one["middle_result_path"])[0] + "/sep/YYMMDDHH.h5"
    middle_result_path_sep = meteva.base.get_path(middle_result_dir_sep,time_fo)
    sta_all = meteva.product.data_prepare.prepare_dataset(para_one)
    df_mid = meteva.perspact.middle_df_sta(sta_all,para_one["mid_method"],grade_list=para_one["grade_list"],compare=para_one["compare"])
    meteva.base.creat_path(middle_result_path_sep)
    df_mid.to_hdf(middle_result_path_sep,"df")


def middle_of_score_surface(para):

    para["base_on"] = "foTime"
    time_cost = time.time()
    middle_result_path = para["middle_result_path"]
    meteva.base.creat_path(middle_result_path)
    model_name_list = list(para["fo_data"].keys())
    mid_method = para["mid_method"]


    mid_list = []
    # 创建一个空的DataFrame，用来记录哪些中间量已经收集
    df1 = pd.DataFrame(data=None, columns=['time', 'dtime', 'member'])
    mid0 = None
    if os.path.exists(middle_result_path):
        if not para["recover"]:
            mid0 = pd.read_hdf(middle_result_path)
            mid_list.append(mid0)
            df1 = mid0[[ "time", "dtime", "member"]]  # 如果有文件就替换掉df1
            df1.drop_duplicates(keep="first", inplace=True)  # 用来记录哪些时次已经收集了中间量

    begin_time = para["begin_time"]
    end_time = para["end_time"]


    v_time_list = []
    for model in model_name_list:
        df1_= df1.loc[df1['member'].isin([model])]
        time1 = begin_time
        para_fo1 = para["fo_data"][model]

        dtime_list = np.arange(para_fo1["dtime"][0], para_fo1["dtime"][1] + 1, para_fo1["dtime"][2]).tolist()

        dtime_list_v = dtime_list
        if para_fo1["operation_para"] is not None:
            d = 0
            if "delta" in  para_fo1["operation_para"].keys():
                d = para_fo1["operation_para"]["delta"]
            elif "span" in para_fo1["operation_para"].keys() :
                d = para_fo1["operation_para"]["span"]

            if d!=0:
                dtime_list.insert(0,0)
                array0 = np.array(dtime_list)
                array1 = array0 + d
                dtime_list_c = list(set((set(array0.tolist())&set(array1.tolist()))))
                dtime_list_c.sort()
                dtime_list_v = dtime_list_c


        while time1 <= end_time:
            df2 = meteva.base.in_time_list(df1_, time1)
            if len(df2.index) ==len(dtime_list_v):
                print(meteva.base.get_path(model + "YYYY年MM月DD日HH时起报的检验中间量已存在",time1))
            else:
                v_time_list.append(time1)


            time1 = time1 + datetime.timedelta(hours=para["time_step"])

    v_time_list = list(set(v_time_list))
    v_time_list.sort()

    if "cpu" not in para.keys():
        cpu = 1
    else:
        cpu = para["cpu"]


    #先删除已有的拆分的中间结果
    dir1,_ =  os.path.split(os.path.abspath(middle_result_path))
    middle_result_dir_sep =dir1+"/sep"
    if os.path.exists(middle_result_dir_sep):
        #删除已有的拆分的中间结果
        file_list = os.listdir(middle_result_dir_sep)
        for file1 in file_list:
            path = middle_result_dir_sep + "/"+ file1
            os.remove(path)
    else:
        pathlib.Path(middle_result_dir_sep).mkdir(parents=True, exist_ok=True)  #创建文件夹

    print(v_time_list)
    #采用多进程统计中间量
    meteva.base.multi_run(cpu,task_of_one_p_surface,para=para,time_fo = v_time_list)

    #将中间量的结果进行合并
    df_list = [mid0]
    file_list = os.listdir(middle_result_dir_sep)
    for file1 in file_list:
        path = middle_result_dir_sep + "/" + file1
        df = pd.read_hdf(path)
        df_list.append(df)
        os.remove(path)

    mid_all = meteva.base.concat(df_list)
    mid_all.sort_values(by=["time","dtime","member"],inplace=True)
    # 先删除文件在重新输出文件，避免文件大小膨胀
    if os.path.exists(middle_result_path):
        os.remove(middle_result_path)
    mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件
    print("中间量拼接程序运行完毕")
    print("拼接后的中间结果已输出至"+middle_result_path)

    print("总耗时："+str(time.time()-time_cost))
    return



if __name__ == "__main__":

    middle_of_score_surface(para_example)
    path = r"H:\test_data\output\mps\na.h5"
    df = pd.read_hdf(path)
    print(df)
    print()

    meteva.base.interp_ss_idw()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun  6 13:41:16 2023

@author: tangbuxing
"""
import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
from sklearn.preprocessing import Binarizer
from scipy.spatial import cKDTree
from meteva.method.space.mode.deltametric import cv2_distanceTransform



def Gbeta_score(ob, fo, threshold, beta = None, alpha = 0):
    out = {}
  
    binarizer=Binarizer(threshold=threshold)
    Z = binarizer.transform(ob)
    Zhat = binarizer.transform(fo)

    #dA = distmap(Z)    #未翻译distmap函数
    #dB = distmap(Zhat)    #未翻译distmap函数
    #dA = cv2.distanceTransform(np.array(~(Z == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
    am = np.array(~(Z == 1) + 0, np.uint8)
    dA = cv2_distanceTransform(am)

    #plt.imshow(dA)
    #dB = cv2.distanceTransform(np.array(~(Zhat == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
    #plt.imshow(dB)
    bm = np.array(~(Zhat == 1) + 0, np.uint8)
    dB = cv2_distanceTransform(bm)

    IA = Z
    IB = Zhat
    if IA.shape != IB.shape:
        print("beta: X and Xhat have different dimensions.")
    N = np.prod(IA.shape) +0.0   # + 0.0 转换成float计算所有元素的乘积
  
    if beta is None:
        beta = N*N   #/4.0
    nA = np.sum(IA)
    nB = np.sum(IB)
    IA_0 = IA ==1
    IB_0 = IB ==1
    nAB = np.sum(IA_0 & IB_0)    #重叠区域
  
    term1 = (nA + nB - 2 * nAB)    #不重叠区域
    term1b = term1/np.sqrt(beta - alpha)   
    term1c = (nB - nAB)/np.sqrt(beta - alpha)
    term1d = (nA - nAB)/np.sqrt(beta - alpha)
    
    medAB = np.sum(dA * IB)/max(1, nB)
    medBA = np.sum(dB * IA)/max(1, nA)
    
    term2 = medAB * nB    
    #term2 = medAB * nB
    term2b = term2/np.sqrt(beta - alpha)
    
    term3 = medBA * nA
    term3b = term3/np.sqrt(beta - alpha)
    
    term4 = (term2 + term3)/np.sqrt(beta - alpha)
    x = term1b * term4
    outAB = np.maximum(1 - (term1c * term2b - alpha), 0)    #pmax -> np.maximum(np.arange(1,6), 3)
    outBA = np.maximum(1 - (term1d * term3b - alpha), 0)
    out = np.maximum(1 - (x - alpha), 0)

    res_data = np.array([[nA, nB, nAB, term1, medAB, medBA, term2, term3, outAB, outBA]])
    # res = pd.DataFrame(res_data, columns=["nA", "nB", "nAB", "nA + nB - 2nAB", "medAB",
    #                                       "medBA", "medAB * nB", "medBA * nA",
    #                                       "GbetaAB", "GbetaBA"])

    out = {'nA':nA, 'nB':nB, 'nAB':nAB, '"nAnB_2nAB':term1, 'medAB':medAB, 'medBA':medBA,
           'medAB_multiply_nB':term2, 'medBA_multiply_nA':term3,
            "GbetaAB":outAB,"GbetaBA":outBA,
           'beta':beta, 'alpha':alpha, 'threshold':threshold}
       
    return out 


if __name__ == '__main__':

    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    res_Gbeta = Gbeta_score(grd_ob.values.squeeze(), grd_fo.values.squeeze(), threshold = 0.1)
    print(res_Gbeta)

    # fcst = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/wrf4ncar0531.RData')['wrf4ncar0531']
    # obs = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/obs0601.RData')['obs0601']
    # res_Gbeta = Gbeta( X = obs.values, Xhat = fcst.values, threshold = 2.1 )
'''
    X = obs.values
    Xhat = fcst.values
    threshold = 2.1
    alpha = 0
    beta = None
    rule = ">"
'''    


    
    
  


import math
import pathlib
import meteva
import datetime
import pandas as pd
import os 
import numpy as np
import  time

para = {
    "mid_method":meteva.method.tase,
    "grade_list": None,
    "compare":None,
    "middle_result_path": r"H:\test_data\output\mps\tase_z.h5",
    "begin_time": datetime.datetime(2018,8,1,0),
    "end_time": datetime.datetime(2018,8,2,0),
    "time_type":"UT",
    "time_step":12,     #起报时间间隔
    "grid" :  meteva.base.grid([70,140,0.25],[10,60,0.25]),  # 检验区域
    "level_list":  [ 850, 700, 500],
    "step":5,  #masker间隔（单位：°）
    "recover":False,
    "cpu":1,
    "ob_data": {
        "CRA40":{
        "dirm": r"\\10.28.16.234\data2\AI\CRA40\2018\Z\LLL\YYYYMMDDHH.nc",  # 实况场数据路径
        "read_method": meteva.base.read_griddata_from_nc,
        "read_para": {},
        "time_type": "UT",  # 实况数据时间类型
        "multiple": 1,  # 当文件数据的单位是位势米，通过乘以9.8转换成位势
        }
    },
    "fo_data": {
        "Fu":{
            "dirm": r"\\10.28.16.234\data2\AI\fuxi\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",
            "dtime": [12, 240, 12],
            "read_method": meteva.base.read_griddata_from_nc,
            "read_para": {},
            "time_type": "UT",  # 预报数据时间类型是北京时，即08时起报
            "multiple": 1,  # 当文件数据的单位是位势米，通过乘以9.8转换成位势
            "move_fo_time":0,
            "veri_by":"CRA40",
            "gh":{
                "dirm": r"\\10.28.16.234\data2\AI\fuxi\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  #位势高度的文件路径
                "read_method": meteva.base.read_griddata_from_nc,     #读取位势高度的函数
                "read_para": {},                                      #读取位势高度所用的函数参数
            },
        },

        "pa": {
            "dirm":   r"\\10.28.16.234\data2\AI\Pangu_ERA5\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",
            "dtime": [12, 240, 12],
            "read_method": meteva.base.read_griddata_from_nc,
            "read_para": {},
            "time_type": "UT",#预报数据时间类型是北京时，即08时起报
            "multiple":1, #当文件数据的单位是位势米，通过乘以9.8转换成位势
            "move_fo_time": 12,
            "veri_by": "CRA40",
            "gh": {
                "dirm": r"\\10.28.16.234\data2\AI\Pangu_ERA5\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc", #位势高度的文件路径
                "read_method": meteva.base.read_griddata_from_nc,    #读取位势高度的函数
                "read_para": {},   #读取位势高度所用的函数参数
            },
        },
    },

    "zs": {
        "path": r"\\10.28.16.234\data2\AI\fuxi\Z\LLL\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",     #地形高度的文件路径
        "read_method": meteva.base.read_griddata_from_nc,   #读取地形高度的函数
        "read_para": {},     #读取地形高度的函数参数
    }
}


def task_of_one_p(para,middle_result_path,v_df,mid0 = None,zs = None):

    time_cost_dict = {"total_cost":0,"veri_cost":0}
    time_cost_total0 = time.time()
    grid0 = para["grid"]
    if para["step"] is None:
        marker = None
    else:
        marker = meteva.perspact.get_grid_marker(grid0, step=para["step"])
    mid_method = para["mid_method"]
    if mid0 is None:
        mid_list = []
    else:
        mid_list = [mid0]
    save_k = 0  # 用来在收集的过程间隔一段时间报错一下结果，避免程序错误时没有任何输出导致要重头再来
    v_df_group, level_list = meteva.base.group(v_df, g="level")
    for i in range(len(level_list)):
        level = level_list[i]
        v_df1 = v_df_group[i]
        v_df_group1, ob_time_list = meteva.base.group(v_df1, g="ob_time")
        for j in range(len(v_df_group1)):
            # 在相同的实况时间分组里，可以重复使用相同的实况
            grd_obs_dict = {}  # 记录读过的实况，ob_time变化时清空

            time_ob = ob_time_list[j]
            time_ob_bt = time_ob + datetime.timedelta(hours=8)
            v_df2 = v_df_group1[j]
            for k in range(len(v_df2.index)):
                time1 = meteva.base.all_type_time_to_datetime(v_df2["time"].values[k])
                dh = v_df2["dtime"].values[k]
                model = v_df2["member"].values[k]
                para_fo1 = para["fo_data"][model]
                move_fo_time = para_fo1["move_fo_time"]
                veri_by = para_fo1["veri_by"]
                if veri_by == "self":
                    veri_by = model

                if veri_by in grd_obs_dict.keys():
                    # 如果已经读过某个名字的实况，就直接取
                    grd_obs = grd_obs_dict[veri_by]
                else:
                    move_fo_time = para_fo1["move_fo_time"]
                    if veri_by == model:  # veri_by == self
                        para_ob1 = para_fo1
                    else:
                        para_ob1 = para["ob_data"][veri_by]

                    if para_ob1["time_type"].lower() == "bt":
                        time_path = time_ob_bt
                    else:
                        time_path = time_ob

                    path0 = meteva.base.get_path(para_ob1["dirm"], time_path)
                    path0 = path0.replace("LLL", str(level))
                    if not os.path.exists(path0):
                        print(path0 + " not exist")
                        continue

                    time_cost0 = time.time()
                    grd_obs = para_ob1["read_method"](path0, grid=para["grid"], time=time_ob,
                                                      dtime=0, data_name="OBS", show=True, level=level,
                                                      **para_ob1["read_para"])
                    time_cost = time.time() - time_cost0
                    if veri_by not in time_cost_dict.keys():
                        time_cost_dict[veri_by] = 0
                    time_cost_dict[veri_by] += time_cost


                    if grd_obs is None:
                        print("faild to read " + path0)
                        continue
                    grd_obs.values *= para_ob1["multiple"]
                    grd_obs_dict[veri_by] = grd_obs

                time1_ = time1
                if para_fo1["time_type"].lower() == "bt":
                    time1_ = time1 + datetime.timedelta(hours=8)
                time1_ -= datetime.timedelta(hours=move_fo_time)
                if para_fo1["dirm"] is not None:
                    path1 = meteva.base.get_path(para_fo1["dirm"], time1_, dh + move_fo_time)
                    path1 = path1.replace("LLL", str(level))
                    time_cost0 = time.time()
                    grd_fo = para_fo1["read_method"](path1, grid=para["grid"], time=time1, dtime=dh, level=level,
                                                     data_name=model, show=True, **para_fo1["read_para"])
                else:
                    grd_fo = para_fo1["read_method"](grid=para["grid"], time=time1, dtime=dh, level=level,
                                                     data_name=model, show=True)
                time_cost = time.time() - time_cost0
                if model not in time_cost_dict.keys():
                    time_cost_dict[model] = 0
                time_cost_dict[model] += time_cost

                if grd_fo is not None:
                    grd_fo.values *= para_fo1["multiple"]
                    time_cost0 = time.time()

                    #读取位势高度用于判断是否在地下
                    if zs is not None:
                        if para_fo1["gh"]["dirm"] is not None:
                            path1 = meteva.base.get_path(para_fo1["gh"]["dirm"], time1_, dh + move_fo_time)
                            path1 = path1.replace("LLL", str(level))
                            time_cost0 = time.time()
                            grd_gh = para_fo1["gh"]["read_method"](path1, grid=para["grid"], time=time1, dtime=dh,
                                                             level=level,
                                                             data_name=model, show=True, **para_fo1["gh"]["read_para"])
                        else:
                            grd_gh = para_fo1["gh"]["read_method"](grid=para["grid"], time=time1, dtime=dh, level=level,
                                                             data_name=model, show=True)

                        grd_fo.values[grd_gh.values<zs.values] = np.nan


                    df_mid = meteva.perspact.middle_df_grd(grd_obs, grd_fo, mid_method, marker=marker,
                                                           grade_list=para["grade_list"], compare=para["compare"])
                    time_cost = time.time() - time_cost0
                    time_cost_dict["veri_cost"] += time_cost

                    mid_list.append(df_mid)
                else:
                    print("faild to read " + path1)

                save_k += 1
                # 当收集了超过500个时效的数据时就输出一次
                if save_k % 500 == 0:
                    mid_all = meteva.base.concat(mid_list)
                    # 先删除文件在重新输出文件，避免文件大小膨胀
                    if os.path.exists(middle_result_path):
                        os.remove(middle_result_path)
                    mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件

    mid_all = meteva.base.concat(mid_list)
    mid_all.sort_values(by=["time", "dtime", "level", "member"], inplace=True)
    # 先删除文件在重新输出文件，避免文件大小膨胀
    if os.path.exists(middle_result_path):
        os.remove(middle_result_path)
    mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件
    print("中间量统计程序运行完毕")
    print("中间结果已输出至" + middle_result_path)
    time_cost_dict["total_cost"] = time.time() - time_cost_total0
    print("单进程耗时："+ str(time_cost_dict["total_cost"]))
    print("检验计算耗时："+str(time_cost_dict["veri_cost"]))
    for key in time_cost_dict:
        if  not key in ["total_cost","veri_cost"]:
            print("读取"+key+"耗时："+str(time_cost_dict[key]))

    return mid_all


def middle_of_score(para):

    time_cost = time.time()
    middle_result_path = para["middle_result_path"]
    meteva.base.creat_path(middle_result_path)
    model_name_list = list(para["fo_data"].keys())
    mid_method = para["mid_method"]
    grid0 = para["grid"]
    # grid0 = meteva.base.grid([0,359.75,0.25],[-89.75,89.75,0.25])
    if para["step"] is None:
        marker = None
    else:
        marker = meteva.perspact.get_grid_marker(grid0, step=para["step"])
    level_list =para["level_list"]
    mid_list = []
    # 创建一个空的DataFrame，用来记录哪些中间量已经收集
    df1 = pd.DataFrame(data=None, columns=['level', 'time', 'dtime', 'member'])
    mid0 = None
    if os.path.exists(middle_result_path):
        if not para["recover"]:
            mid0 = pd.read_hdf(middle_result_path)
            mid_list.append(mid0)
            df1 = mid0[["level", "time", "dtime", "member"]]  # 如果有文件就替换掉df1
            df1.drop_duplicates(keep="first", inplace=True)  # 用来记录哪些时次已经收集了中间量

    begin_time = para["begin_time"]
    end_time = para["end_time"]
    if para["time_type"].lower()=="bt":
        begin_time -= datetime.timedelta(hours=8)
        end_time -= datetime.timedelta(hours=8)

    v_member_list = []
    v_time_list = []
    v_dtime_list = []
    v_level_list = []
    for model in model_name_list:
        df1_= df1.loc[df1['member'].isin([model])]
        time1 = begin_time
        para_fo1 = para["fo_data"][model]

        dtime_list = np.arange(para_fo1["dtime"][0], para_fo1["dtime"][1] + 1, para_fo1["dtime"][2]).tolist()

        while time1 <= end_time:
            df2 = meteva.base.in_time_list(df1_, time1)
            if len(df2.index) ==len(dtime_list):
                print(meteva.base.get_path(model + "YYYY年MM月DD日HH时起报的检验中间量已存在",time1))
            else:
                for dh in dtime_list:
                    df3 = meteva.base.in_dtime_list(df2, dtime_list=dh)
                    for level in level_list:
                        df4 = meteva.base.in_level_list(df3, level_list=level)
                        if len(df4.index) ==0:
                            v_member_list.append(model)
                            v_time_list.append(time1)
                            v_dtime_list.append(dh)
                            v_level_list.append(level)
            time1 = time1 + datetime.timedelta(hours=para["time_step"])

    v_df = pd.DataFrame({
        "level":v_level_list,
        "time":v_time_list,
        "dtime":v_dtime_list,
        "member":v_member_list
    })

    zs = None
    #读取地形高度
    if "zs" in para.keys():
        if "read_method" in para["zs"]:
            if para["zs"]["read_method"] is not None:
                zs = para["zs"]["read_method"](para["zs"]["path"],grid = para["grid"],**para["zs"]["read_para"])
                if zs is None:
                    print("地形高度数据读取失败，请检查地形数据文件是否存在，地形数据读取函数和地形数据格式是否吻合")

    if "cpu" not in para.keys() or para["cpu"] ==1:
        task_of_one_p(para,middle_result_path,v_df,mid0 = mid0,zs= zs)

    else:
        #多进程并行方案

        #先删除已有的拆分的中间结果
        dir1,_ =  os.path.split(os.path.abspath(middle_result_path))
        middle_result_dir_sep =dir1+"/sep"
        if os.path.exists(middle_result_dir_sep):
            #删除已有的拆分的中间结果
            file_list = os.listdir(middle_result_dir_sep)
            for file1 in file_list:
                path = middle_result_dir_sep + "/"+ file1
                os.remove(path)
        else:
            pathlib.Path(middle_result_dir_sep).mkdir(parents=True, exist_ok=True)  #创建文件夹


        v_df["ob_time"] = v_df["time"].astype("datetime64[us]") + v_df["dtime"]*np.timedelta64(1,"h")
        v_df.sort_values(by=["level","ob_time"],inplace=True)  #按层次、实况时间排序，可以尽可能让共用实况的任务靠近
        ntask = len(v_df.index)
        cpu = para["cpu"]

        task_of_one_cpu = int(math.ceil(ntask/cpu))
        middle_result_path_list = []
        v_df_list = []
        for i in range(cpu):
            index_s = task_of_one_cpu*(i)
            index_e = task_of_one_cpu*(i+1)
            df1 = v_df.iloc[index_s:index_e,:]
            v_df_list.append(df1)
            middle_result_path_list.append(middle_result_dir_sep+"/"+str(i)+".h5")

        #采用多进程统计中间量
        meteva.base.multi_run(cpu,task_of_one_p,para=para,middle_result_path=middle_result_path_list,v_df = v_df_list,zs = zs)

        #将中间量的结果进行合并
        df_list = [mid0]
        file_list = os.listdir(middle_result_dir_sep)
        for file1 in file_list:
            path = middle_result_dir_sep + "/" + file1
            df = pd.read_hdf(path)
            df_list.append(df)
            os.remove(path)

        mid_all = meteva.base.concat(df_list)
        mid_all.sort_values(by=["time","dtime","level","member"],inplace=True)
        # 先删除文件在重新输出文件，避免文件大小膨胀
        if os.path.exists(middle_result_path):
            os.remove(middle_result_path)
        mid_all.to_hdf(middle_result_path, "df")  # 将结果输出到文件
        print("中间量拼接程序运行完毕")
        print("拼接后的中间结果已输出至"+middle_result_path)

    print("总耗时："+str(time.time()-time_cost))
    return



if __name__ == "__main__":


    pass


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May  1 19:01:40 2023

@author: tangbuxing
"""
import numpy as np
import meteva.method as mem

#import locperf


def distob(X, Y, distfun = "distmapfun"):
    out = {}
    nX = np.sum(np.sum(X, axis = 0))
    nY = np.sum(np.sum(Y, axis = 0))
    
    if nX == 0 and nY == 0:
        return 0 
    elif nX == 0 or nY == 0:
        return np.max(np.shape(np.matrix(X)))
    else:
        out = mem.mode.locperf.locperf(X = X, Y = Y, which_stats = "med")["medMiss"]
        
    return out

